package org.codehaus.xfire.picocontainer.example.server;

import java.io.IOException;

import org.codehaus.xfire.XFire;
import org.codehaus.xfire.XFireFactory;
import org.codehaus.xfire.picocontainer.DefaultServiceRegistryDelegator;
import org.codehaus.xfire.picocontainer.DefaultTransportManagerDelegator;
import org.codehaus.xfire.picocontainer.ObjectServiceComponentAdapter;
import org.codehaus.xfire.picocontainer.XFireServiceRegisterVisitor;
import org.codehaus.xfire.service.MessageService;
import org.codehaus.xfire.service.Service;
import org.codehaus.xfire.service.ServiceFactory;
import org.codehaus.xfire.service.ServiceRegistry;
import org.codehaus.xfire.service.object.ObjectServiceFactory;
import org.codehaus.xfire.transport.TransportManager;
import org.codehaus.xfire.type.DefaultTypeMappingRegistry;
import org.codehaus.xfire.type.TypeMappingRegistry;
import org.mortbay.http.HttpContext;
import org.mortbay.http.HttpServer;
import org.mortbay.jetty.servlet.ServletHandler;
import org.mortbay.util.MultiException;
import org.picocontainer.MutablePicoContainer;
import org.picocontainer.Parameter;
import org.picocontainer.PicoVisitor;
import org.picocontainer.defaults.CachingComponentAdapter;
import org.picocontainer.defaults.ConstantParameter;
import org.picocontainer.defaults.ConstructorInjectionComponentAdapter;
import org.picocontainer.defaults.DefaultPicoContainer;

public class Main {

    private static HttpServer server;

    public static void main(String[] args) {
        try {

            MutablePicoContainer pico = new DefaultPicoContainer();
            populateXFireComponents(pico);
            pico.registerComponent(new CachingComponentAdapter(new ObjectServiceComponentAdapter(new ConstructorInjectionComponentAdapter(EchoService.class, EchoService.class))));

            Service service = createACustomServiceUsingServiceFactory((ServiceFactory) pico.getComponentInstance(ServiceFactory.class));
            pico.registerComponentInstance(service);

            pico.start();

            pico.accept((PicoVisitor) pico.getComponentInstance(XFireServiceRegisterVisitor.class));

            startXFireServer();

            while (true) {
                // I hope to get a "Press enter key to stop the server..." here.
                if (1 == 2) { // Just to break the "unrechable code" error...
                    break;
                }
                Thread.sleep(600);
            }

            stopXFireServer();

        } catch (Exception e) {
            // Do not do it at home.
            throw new RuntimeException(e);
        }
    }

    private static void populateXFireComponents(MutablePicoContainer pico) throws Exception {
        XFire xfire = XFireFactory.newInstance().getXFire();
        pico.registerComponentInstance(XFire.class, xfire);
        pico.registerComponentImplementation(TransportManager.class, DefaultTransportManagerDelegator.class);
        pico.registerComponentImplementation(ServiceRegistry.class, DefaultServiceRegistryDelegator.class);
        pico.registerComponentImplementation(ServiceFactory.class, ObjectServiceFactory.class);
        pico.registerComponentImplementation(TypeMappingRegistry.class, DefaultTypeMappingRegistry.class, new Parameter[] { new ConstantParameter(new Boolean(true)) });
        pico.registerComponentImplementation(XFireServiceRegisterVisitor.class);
    }

    private static Service createACustomServiceUsingServiceFactory(ServiceFactory serviceFactory) {
        MessageService service = (MessageService) serviceFactory.create(EchoService.class);
        service.setName("Echo2Service");
        return service;
    }

    private static void startXFireServer() throws IOException, MultiException {
        server = new HttpServer();
        server.addListener(":8080");
        HttpContext context = server.getContext("/");

        ServletHandler handler = new ServletHandler();
        handler.addServlet("XFireServlet", "/xfire/*", "org.codehaus.xfire.transport.http.XFireServlet");
        context.addHandler(handler);

        server.start();
    }

    private static void stopXFireServer() throws InterruptedException {
        server.stop();
        server = null;
    }
}
